#include<LayOutAPI\layout.h>
#include<SketchUpAPI\sketchup.h>
#include <assert.h>
#include<string>
#include<cstdio>
#include<vector>
#include"threadpool.h"

#define SU(api_function_call) {\
  SUResult su_api_result = api_function_call;\
  assert(SU_ERROR_NONE == su_api_result);\
}\

std::mutex _lockAddEntity;

int main() {
	std::string skp_file = "F:/Desktop/relation_skp/test3.skp";

	LOInitialize();
	LODocumentRef document = SU_INVALID;
	SU(LODocumentCreateEmpty(&document));

	LOLayerRef layer = SU_INVALID;
	SU(LODocumentGetLayerAtIndex(document, 0, &layer));

	LOPageRef page = SU_INVALID;
	SU(LODocumentGetPageAtIndex(document, 0, &page));

	ThreadPool pool(4);

	std::vector<std::future<int>> executors;
	// test3.skp have 43 pages.  page1 - page43
	int num_of_out_page = 4; // It's ok when num_of_out_page=4.
	for (int i = 0; i < num_of_out_page; i++) {
		executors.emplace_back(pool.enqueue([&document, skp_file, i] {

			std::string page_name = "page";
			char num_str[3] = { 0 };
			itoa(i + 1, num_str, 10);
			page_name.append(num_str);

			LOAxisAlignedRect2D rect;  // (0,0)  (5,5)
			rect.upper_left.x = rect.upper_left.y = 0;
			rect.lower_right.x = rect.lower_right.y = 5;
			LOSketchUpModelRef model = SU_INVALID;

			SUResult res = LOSketchUpModelCreate(&model, skp_file.c_str(), &rect);
			if (res == SU_ERROR_NONE) {
				std::unique_lock<std::mutex> lock{ _lockAddEntity };  // lock AddEntity

				res = LODocumentAddEntityUsingIndexes(document, LOSketchUpModelToEntity(model), 0, 0);
				if (res == SU_ERROR_NONE) {
					res = LOSketchUpModelSetCurrentScene(model, i + 1);
					if (res == SU_ERROR_NONE)
						printf("%d %s ok\n", i, page_name.c_str());
					else
						printf("%d %s SetScene Error %d\n", i, page_name.c_str(), res);
				}
				else
					printf("%d %s AddEntity Error %d\n", i, page_name.c_str(), res);

				lock.unlock();  // unlock AddEntity

				SU(LOSketchUpModelRelease(&model));  // release SketchUpModel
			}
			else {
				printf("%d %s Create Error %d\n", i, page_name.c_str(), res);
			}
			return i;
		}));
	}

	// main thread wait all executor
	for (auto && executor : executors)
		executor.get();

	SU(LODocumentSaveToFile(document, "F:/Desktop/relation_skp/test.layout", LODocumentVersion_Current));
	SU(LODocumentRelease(&document));

	LOTerminate();
	printf("finish\n");
	return 0;
}